#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from pathlib import Path

from axosyslog_light.driver_io.unix_dgram.unix_dgram_io import UnixDgramIO
from axosyslog_light.syslog_ng_config.statements.sources.source_driver import SourceDriver
from axosyslog_light.syslog_ng_ctl.legacy_stats_handler import LegacyStatsHandler
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler


class UnixDgramSource(SourceDriver):
    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        path: Path,
        **options,
    ) -> None:
        self.io = UnixDgramIO(path)

        self.driver_name = "unix-dgram"
        super(UnixDgramSource, self).__init__(stats_handler, prometheus_stats_handler, [path], options=options)

    def write_log(self, message):
        self.io.write_messages([message])

    def write_logs(self, messages):
        self.io.write_messages(messages)

    def close_socket(self):
        self.io.close_socket()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from pathlib import Path

from axosyslog_light.common.pytest_operations import calculate_testcase_name


INSTANCE_PATH = None


class TestcaseParameters(object):
    def __init__(self, pytest_request):
        testcase_name = calculate_testcase_name(pytest_request.node.name)
        absolute_framework_dir = Path(__file__).parents[3]
        self.testcase_parameters = {
            "dirs": {
                "shared_dir": Path(absolute_framework_dir, "shared_files"),
                "testcase_dir": Path(pytest_request.fspath).parents[0],
            },
            "file_paths": {
                "testcase_file": Path(pytest_request.fspath),
            },
            "testcase_name": testcase_name,
            "external_tool": pytest_request.config.getoption("run_under"),
        }

        try:
            install_dir = pytest_request.config.getoption("installdir")
            self.testcase_parameters["dirs"]["install_dir"] = Path(install_dir)
        except TypeError:
            pass

    def get_install_dir(self):
        return self.testcase_parameters["dirs"]["install_dir"]

    def get_shared_dir(self):
        return self.testcase_parameters["dirs"]["shared_dir"]

    def get_testcase_dir(self):
        return self.testcase_parameters["dirs"]["testcase_dir"]

    def get_testcase_file(self):
        return self.testcase_parameters["file_paths"]["testcase_file"]

    def get_testcase_name(self):
        return self.testcase_parameters["testcase_name"]

    def get_external_tool(self):
        return self.testcase_parameters["external_tool"]



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import logging
import shutil
from pathlib import Path

from axosyslog_light.common.blocking import DEFAULT_TIMEOUT
from axosyslog_light.common.blocking import wait_until_true
from axosyslog_light.common.blocking import wait_until_true_custom

logger = logging.getLogger(__name__)


def open_file(file_path, mode):
    return open(file_path, mode, errors="backslashreplace")


def copy_file(src_file_path, dst_dir):
    shutil.copy(src_file_path, dst_dir)


def copy_shared_file(testcase_parameters, shared_file_name):
    shared_dir = testcase_parameters.get_shared_dir()
    copy_file(Path(shared_dir, shared_file_name), Path.cwd())
    return Path(Path.cwd(), shared_file_name)


def get_shared_file(shared_file_name):
    absolute_framework_dir = Path(__file__).parents[3].resolve()
    return absolute_framework_dir / "shared_files" / shared_file_name


def delete_session_file(shared_file_name):
    shared_file_name = Path(shared_file_name)
    shared_file_name.unlink()


class File(object):
    def __init__(self, file_path):
        self.path = Path(file_path)
        self.__opened_file = None

    def wait_for_creation(self):
        file_created = wait_until_true(self.path.exists)
        if file_created:
            logger.debug("File has been created: {}".format(self.path))
        else:
            raise Exception("File was not created in time: {}".format(self.path))
        return file_created

    def open(self, mode):
        self.__opened_file = open_file(self.path, mode)
        return self.__opened_file

    def close(self):
        if self.is_opened():
            self.__opened_file.close()
            self.__opened_file = None

    def is_opened(self):
        return self.__opened_file is not None

    def readline(self):
        if not self.is_opened():
            raise Exception("File was not opened before trying to read from it.")
        return self.__opened_file.readline()

    def read_full_line(self, timeout=DEFAULT_TIMEOUT):
        def read_full_line_segmented(f, line_segments):
            line_read = f.readline()
            if not line_read:
                if len(line_segments) > 0:
                    # Has to be called again to collect the rest of the line
                    return False
                # EOF reached immediately
                return True

            line_segments.append(line_read)
            return line_read[-1] == "\n"

        if not self.is_opened():
            raise Exception("File was not opened before trying to read from it.")

        line_segments = []
        wait_until_true_custom(read_full_line_segmented, (self.__opened_file, line_segments), timeout=timeout, poll_freq=0)
        return "".join(line_segments)

    def read(self):
        content = ""
        while True:
            buffer = self.__opened_file.read()
            if buffer == "":
                break
            content += buffer
        return content

    def write(self, content):
        self.__opened_file.write(content)
        self.__opened_file.flush()

    def write_content_and_close(self, content):
        if not self.is_opened():
            self.open(mode="w+")
        self.write(content)
        self.close()

    def wait_for_lines(self, lines, timeout=DEFAULT_TIMEOUT):
        def find_lines_in_file(lines_to_find, lines_found, f):
            line_read = self.read_full_line(timeout=timeout)
            if not line_read and lines_to_find:
                return False
            for line_to_find in lines_to_find:
                if line_to_find in line_read:
                    lines_found.append(line_read)
                    lines_to_find.remove(line_to_find)
                    break
            everything_is_found = lines_to_find == []
            return everything_is_found

        lines_found = []
        if not wait_until_true_custom(find_lines_in_file, (lines, lines_found, self), timeout=timeout, poll_freq=0):
            raise Exception("Could not find all lines in {}. Remaining lines to find: {} Lines found: {}".format(self.path, lines, lines_found))

        return lines_found

    def wait_for_number_of_lines(self, number_of_lines, timeout=DEFAULT_TIMEOUT):
        lines = ["\n"] * number_of_lines
        try:
            return self.wait_for_lines(lines, timeout)
        except Exception:
            raise Exception("Could not find {} number of lines in {}. Remaining number of lines: {}.".format(number_of_lines, self.path, len(lines)))



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from pathlib import Path

from axosyslog_light.common.asynchronous import BackgroundEventLoop
from axosyslog_light.common.blocking import DEFAULT_TIMEOUT
from axosyslog_light.common.network import SingleConnectionUnixStreamServer
from axosyslog_light.driver_io import message_readers
from axosyslog_light.syslog_ng_config.statements.destinations.destination_driver import DestinationDriver
from axosyslog_light.syslog_ng_ctl.legacy_stats_handler import LegacyStatsHandler
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler


class UnixStreamDestination(DestinationDriver):
    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        file_name: str,
        **options,
    ) -> None:
        self.driver_name = "unix-stream"
        self.path = Path(file_name)

        self.__server = None
        self.__message_reader = None

        super(UnixStreamDestination, self).__init__(stats_handler, prometheus_stats_handler, [self.path], options)

    def start_listener(self):
        self.__server = SingleConnectionUnixStreamServer(self.path)
        self.__message_reader = message_readers.SingleLineStreamReader(self.__server)
        BackgroundEventLoop().wait_async_result(self.__server.start(), timeout=DEFAULT_TIMEOUT)

    def stop_listener(self):
        if self.__message_reader is not None:
            BackgroundEventLoop().wait_async_result(self.__server.stop(), timeout=DEFAULT_TIMEOUT)
            self.__message_reader = None
            self.__server = None

    def read_log(self, timeout=DEFAULT_TIMEOUT):
        return self.read_logs(1, timeout)[0]

    def read_logs(self, counter, timeout=DEFAULT_TIMEOUT):
        return self.__message_reader.wait_for_number_of_messages(counter, timeout)

    def read_until_logs(self, logs, timeout=DEFAULT_TIMEOUT):
        return self.__message_reader.wait_for_messages(logs, timeout)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
# Copyright (c) 2020 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from __future__ import annotations

from pathlib import Path

from axosyslog_light.executors.process_executor import ProcessExecutor
from axosyslog_light.helpers.loggen.loggen_executor import LoggenExecutor
from axosyslog_light.helpers.loggen.loggen_executor import LoggenStartParams
from psutil import Popen


class LoggenLocalExecutor(LoggenExecutor):
    def __init__(self, loggen_binary_path: Path) -> None:
        self.__binary_path = loggen_binary_path
        super().__init__()

    def _start(self, start_params: LoggenStartParams, stderr: Path, stdout: Path, instance_index: int) -> Popen:
        return ProcessExecutor().start([self.__binary_path] + start_params.format(), stdout, stderr)

    def _copy(self) -> LoggenExecutor:
        return LoggenLocalExecutor(self.__binary_path)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import logging

import psutil
from axosyslog_light.common.file import File

logger = logging.getLogger(__name__)


def prepare_std_outputs(stdout_path, stderr_path):
    stdout = File(stdout_path)
    stderr = File(stderr_path)
    return stdout, stderr


def prepare_printable_command(command):
    printable_command = ""
    for command_arg in command:
        printable_command += "{} ".format(str(command_arg))
    return printable_command


def prepare_executable_command(command):
    executable_command = []
    for command_arg in command:
        executable_command.append(str(command_arg))
    return executable_command


class CommandExecutor(object):
    def __init__(self):
        self.__start_timeout = 10

    def run(self, command, stdout_path, stderr_path):
        printable_command = prepare_printable_command(command)
        executable_command = prepare_executable_command(command)
        stdout, stderr = prepare_std_outputs(stdout_path, stderr_path)
        logger.debug("The following command will be executed:\n{}\n".format(printable_command))
        cmd = psutil.Popen(executable_command, stdout=stdout.open(mode="w"), stderr=stderr.open(mode="w"))
        exit_code = cmd.wait(timeout=self.__start_timeout)

        stdout.close()
        stderr.close()
        stdout_content, stderr_content = self.__process_std_outputs(stdout, stderr)
        self.__process_exit_code(printable_command, exit_code, stdout_content, stderr_content)
        return {"exit_code": exit_code, "stdout": stdout_content, "stderr": stderr_content}

    def __process_std_outputs(self, stdout, stderr):
        stdout.open("r")
        stdout_content = stdout.read()
        stdout.close()

        stderr.open("r")
        stderr_content = stderr.read()
        stderr.close()

        return stdout_content, stderr_content

    def __process_exit_code(self, command, exit_code, stdout, stderr):
        exit_code_debug_log = "\nExecuted command: {}\n\
Exit code: {}\n\
Stdout: {}\n\
Stderr: {}\n".format(command.rstrip(), exit_code, stdout.rstrip(), stderr.rstrip())
        logger.debug(exit_code_debug_log)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import MetricFilter
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler


class LogPath(object):
    def __init__(self, prometheus_stats_handler: PrometheusStatsHandler, name=None):
        self.__group_type = "log"
        self.__name = name
        self.__logpath = []
        self.__flags = []

        self.__metric_filters = []
        if name:
            self.__metric_filters += [
                MetricFilter("syslogng_route_ingress_total", {"id": name}),
                MetricFilter("syslogng_route_egress_total", {"id": name}),
            ]

        self.__prometheus_stats_handler = prometheus_stats_handler

    @property
    def group_type(self):
        return self.__group_type

    @property
    def name(self):
        return self.__name

    @property
    def logpath(self):
        return self.__logpath

    @property
    def flags(self):
        return self.__flags

    def add_group(self, group):
        self.logpath.append(group)

    def add_groups(self, groups):
        for group in groups:
            self.add_group(group)

    def add_flag(self, flag):
        self.flags.append(flag)

    def add_flags(self, flags):
        for flag in flags:
            self.add_flag(flag)

    def get_prometheus_stats(self):
        return self.__prometheus_stats_handler.get_samples(self.__metric_filters)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2020 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from axosyslog_light.driver_io.network.network_io import NetworkIO
from axosyslog_light.syslog_ng_config.statements.sources.source_driver import SourceDriver
from axosyslog_light.syslog_ng_ctl.legacy_stats_handler import LegacyStatsHandler
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler


def map_transport(transport):
    mapping = {
        "auto": NetworkIO.Transport.AUTO,
        "tcp": NetworkIO.Transport.TCP,
        "udp": NetworkIO.Transport.UDP,
        "tls": NetworkIO.Transport.TLS,
        "proxied-tcp": NetworkIO.Transport.PROXIED_TCP,
        "proxied-tls": NetworkIO.Transport.PROXIED_TLS,
        "proxied-tls-passthrough": NetworkIO.Transport.PROXIED_TLS_PASSTHROUGH,
    }
    transport = transport.replace("_", "-").replace("'", "").replace('"', "").lower()

    return mapping[transport]


def create_io(options):
    ip = options["ip"] if "ip" in options else "localhost"
    transport = options["transport"] if "transport" in options else "tcp"

    return NetworkIO(ip, options["port"], map_transport(transport), True)


class SyslogSource(SourceDriver):
    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        **options,
    ) -> None:
        self.io = create_io(options)

        self.driver_name = "syslog"
        super(SyslogSource, self).__init__(stats_handler, prometheus_stats_handler, options=options)

    def write_log(self, message, rate=None, transport=None, framed=None, client_port=0):
        self.io.write_messages([message], rate=rate, transport=transport, framed=framed, client_port=client_port)

    def write_logs(self, messages, rate=None, transport=None, framed=None, client_port=0):
        self.io.write_messages(messages, rate=rate, transport=transport, framed=framed, client_port=client_port)

    def write_logs_with_proxy_header(self, proxy_version, src_ip, dst_ip, src_port, dst_port, messages, rate=None, transport=None, framed=None, client_port=0):
        self.io.write_messages_with_proxy_header(proxy_version, src_ip, dst_ip, src_port, dst_port, messages, rate=rate, transport=transport, framed=framed, client_port=client_port)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
# flake8: noqa: F401, F811
from axosyslog_light.driver_io.file.file_io import FileIO
from axosyslog_light.self_test_fixtures import temp_file
from axosyslog_light.self_test_fixtures import test_message


def test_file_io_write_read(temp_file, test_message):
    fileio = FileIO(temp_file)
    fileio.write_message(test_message)
    output = fileio.read_number_of_messages(1)
    assert [test_message] == output


def test_file_io_multiple_write_read(temp_file, test_message):
    fileio = FileIO(temp_file)
    fileio.write_message(test_message)
    fileio.write_message(test_message)
    output = fileio.read_number_of_messages(2)
    assert [test_message, test_message] == output



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from __future__ import annotations

import pickle
import typing
from pathlib import Path

from filelock import FileLock


class SessionData:
    SINGLETON: typing.Optional[SessionData] = None
    SESSION_FILE = Path("axosyslog-light.session.pickle").resolve().absolute()
    SESSION_LOCK_FILE = Path("axosyslog-light.lock").resolve().absolute()

    def __init__(self) -> None:
        if SessionData.SINGLETON is not None:
            raise ValueError("SessionData is a singleton, cannot create more than one instance")
        SessionData.SINGLETON = self

        self.__data: typing.Dict[str, typing.Any] = dict()
        self.__lock = FileLock(SessionData.SESSION_LOCK_FILE)

        with self.__lock:
            self.__sync_from_file()

    @staticmethod
    def get_singleton() -> SessionData:
        if SessionData.SINGLETON is None:
            SessionData.SINGLETON = SessionData()
        return SessionData.SINGLETON

    @staticmethod
    def cleanup() -> None:
        if get_session_data().__lock.is_locked:
            raise Exception("SessionData.cleanup() must be used without the 'with' statement")

        SessionData.SESSION_FILE.unlink()
        SessionData.SESSION_LOCK_FILE.unlink()

    def __sync_from_file(self) -> None:
        if not SessionData.SESSION_FILE.exists():
            return

        with SessionData.SESSION_FILE.open("rb") as f:
            self.__data.update(pickle.load(f))

    def __sync_to_file(self) -> None:
        with SessionData.SESSION_FILE.open("wb") as f:
            pickle.dump(self.__data, f)

    def __enter__(self) -> SessionData:
        self.__lock.acquire()
        self.__sync_from_file()
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        self.__sync_to_file()
        self.__lock.release()

    def __ensure_with(self) -> None:
        if not self.__lock.is_locked:
            raise ValueError("SessionData must be used with the 'with' statement")

    def __getitem__(self, key: str) -> typing.Any:
        self.__ensure_with()
        return self.__data[key]

    def __setitem__(self, key: str, value: typing.Any) -> None:
        self.__ensure_with()
        self.__data[key] = value

    def get(self, key: str, default: typing.Any = None) -> typing.Any:
        self.__ensure_with()
        return self.__data.get(key, default)

    def clear(self) -> None:
        self.__ensure_with()
        self.__data.clear()


def get_session_data() -> SessionData:
    return SessionData.get_singleton()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import typing
from pathlib import Path

from axosyslog_light.executors.command_executor import CommandExecutor
from axosyslog_light.syslog_ng_ctl.syslog_ng_ctl_executor import SyslogNgCtlExecutor


class SyslogNgCtlLocalExecutor(SyslogNgCtlExecutor):
    def __init__(
        self,
        syslog_ng_ctl_binary_path: Path,
        syslog_ng_control_socket_path: Path,
    ) -> None:
        self.__syslog_ng_ctl_binary_path = syslog_ng_ctl_binary_path
        self.__syslog_ng_control_socket_path = syslog_ng_control_socket_path
        self.__command_executor = CommandExecutor()

    def run_command(
        self,
        instance_name: str,
        command_short_name: str,
        command: typing.List[str],
    ) -> typing.Dict[str, typing.Any]:
        ctl_command = [self.__syslog_ng_ctl_binary_path]
        ctl_command += command
        ctl_command.append("--control={}".format(self.__syslog_ng_control_socket_path))

        return self.__command_executor.run(
            command=ctl_command,
            stdout_path=self.construct_std_file_path(instance_name, command_short_name, "stdout"),
            stderr_path=self.construct_std_file_path(instance_name, command_short_name, "stderr"),
        )



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2019 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from pathlib import Path
from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement
from xml.etree.ElementTree import tostring

from axosyslog_light.syslog_ng_config.statements.parsers.parser import Parser
from axosyslog_light.syslog_ng_ctl.legacy_stats_handler import LegacyStatsHandler
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler


class DBParserConfig(object):
    def __init__(self, ruleset_pattern, rules):
        self.ruleset_pattern = ruleset_pattern
        self.rules = rules

    def write_to(self, file_name):
        with file_name.open("wb") as f:
            node_patterndb = Element("patterndb", version="5")
            node_ruleset = SubElement(node_patterndb, "ruleset", name="some_name", id="1234")
            node_ruleset_pattern = SubElement(node_ruleset, "pattern")
            node_ruleset_pattern.text = self.ruleset_pattern
            node_rules = SubElement(node_ruleset, "rules")

            rule_id = 0
            for rule in self.rules:
                node_rule = SubElement(node_rules, "rule", id=str(rule_id))
                node_rule.set("class", rule["class"])
                rule_id += 1
                node_patterns = SubElement(node_rule, "patterns")
                node_pattern = SubElement(node_patterns, "pattern")
                node_pattern.text = rule["rule"]

            f.write(tostring(node_patterndb))


class DBParser(Parser):
    index = 0

    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        config: DBParserConfig,
        **options,
    ) -> None:
        path = Path("patterndb-{}.xml".format(self.index))
        config.write_to(path)
        self.index += 1
        super(DBParser, self).__init__(
            "db-parser",
            stats_handler=stats_handler,
            prometheus_stats_handler=prometheus_stats_handler,
            file=path.resolve(),
            **options,
        )



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from axosyslog_light.common.file import File


class FileIO():
    def __init__(self, file_path):
        self.__readable_file = File(file_path)
        self.__writeable_file = File(file_path)

    def read_number_of_messages(self, counter):
        if not self.__readable_file.is_opened():
            if not self.__readable_file.wait_for_creation():
                raise Exception("{} was not created in time.".format(self.__readable_file.path))
            self.__readable_file.open("r")

        return [line.rstrip("\n") for line in self.__readable_file.wait_for_number_of_lines(counter)]

    def read_until_messages(self, lines):
        if not self.__readable_file.is_opened():
            if not self.__readable_file.wait_for_creation():
                raise Exception("{} was not created in time.".format(self.__readable_file.path))
            self.__readable_file.open("r")

        return [line.rstrip("\n") for line in self.__readable_file.wait_for_lines(lines)]

    def read_all(self):
        if not self.__readable_file.is_opened():
            if not self.__readable_file.wait_for_creation():
                raise Exception("{} was not created in time.".format(self.__readable_file.path))
            self.__readable_file.open("r")

        return self.__readable_file.read().splitlines()

    def write_raw(self, raw_content):
        if not self.__writeable_file.is_opened():
            self.__writeable_file.open("a+")

        self.__writeable_file.write_content_and_close(raw_content)

    def write_message(self, message):
        self.write_raw(message + "\n")

    def write_messages(self, messages):
        self.write_raw("".join([message + "\n" for message in messages]))

    def close_readable_file(self):
        self.__readable_file.close()

    def close_writeable_file(self):
        self.__writeable_file.close()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from axosyslog_light.executors.process_executor import ProcessExecutor


def test_start_stop_process(tmpdir):
    stdout_file = tmpdir.join("stdout.log")
    stderr_file = tmpdir.join("stderr.log")
    process_executor = ProcessExecutor()
    process_command = ["ls"]
    process_executor.start(process_command, stdout_file, stderr_file)
    assert process_executor.process is not None
    assert process_executor.process.pid is not None



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import typing
from abc import ABC
from abc import abstractmethod
from dataclasses import dataclass
from pathlib import Path
from subprocess import Popen


@dataclass
class SyslogNgStartParams:
    foreground: bool = True
    enable_core: bool = True
    stderr: bool = True
    debug: bool = True
    trace: bool = True
    verbose: bool = True
    startup_debug: bool = True
    no_caps: bool = True
    syntax_only: bool = False
    version: bool = False
    config_path: typing.Optional[Path] = None
    persist_path: typing.Optional[Path] = None
    pid_path: typing.Optional[Path] = None
    control_socket_path: typing.Optional[Path] = None
    preprocess_into: typing.Optional[Path] = None

    def format(self) -> typing.List[str]:
        params = []
        if self.foreground:
            params += ["--foreground"]
        if self.enable_core:
            params += ["--enable-core"]
        if self.stderr:
            params += ["--stderr"]
        if self.debug:
            params += ["--debug"]
        if self.trace:
            params += ["--trace"]
        if self.verbose:
            params += ["--verbose"]
        if self.startup_debug:
            params += ["--startup-debug"]
        if self.no_caps:
            params += ["--no-caps"]
        if self.syntax_only:
            params += ["--syntax-only"]
        if self.version:
            params += ["--version"]
        if self.config_path is not None:
            params += [f"--cfgfile={self.config_path}"]
        if self.persist_path is not None:
            params += [f"--persist-file={self.persist_path}"]
        if self.pid_path is not None:
            params += [f"--pidfile={self.pid_path}"]
        if self.control_socket_path is not None:
            params += [f"--control={self.control_socket_path}"]
        if self.preprocess_into is not None:
            params += [f"--preprocess-into={self.preprocess_into}"]
        return params


class SyslogNgExecutor(ABC):
    @abstractmethod
    def run_process(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
    ) -> Popen:
        pass

    @abstractmethod
    def run_process_with_valgrind(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
        valgrind_output_path: Path,
    ) -> Popen:
        pass

    @abstractmethod
    def run_process_with_gdb(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
    ) -> Popen:
        pass

    @abstractmethod
    def run_process_with_strace(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
        strace_output_path: Path,
    ) -> Popen:
        pass

    @abstractmethod
    def get_backtrace_from_core(
        self,
        core_file_path: Path,
        stderr_path: Path,
        stdout_path: Path,
    ) -> typing.Dict[str, typing.Any]:
        pass



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import time

DEFAULT_TIMEOUT = 20  # 20 sec
POLL_FREQ = 0.001  # 1 millisecond


def wait_until_true(func, *args):
    return wait_until_true_custom(func, args)


def wait_until_true_custom(func, args=(), timeout=DEFAULT_TIMEOUT, poll_freq=POLL_FREQ):
    t_end = time.monotonic() + timeout
    while time.monotonic() <= t_end:
        result = func(*args)
        if result:
            return result
        time.sleep(poll_freq)
    return result


def wait_until_false(func, *args):
    return wait_until_false_custom(func, args)


def wait_until_false_custom(func, args=(), timeout=DEFAULT_TIMEOUT):
    def negate(func, args):
        return not func(*args)
    return wait_until_true_custom(negate, (func, args), timeout=timeout)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import typing
from abc import ABC
from abc import abstractmethod
from enum import Enum
from pathlib import Path


class QueryTypes(Enum):
    QUERY_GET = 0
    QUERY_SUM = 1
    QUERY_LIST = 2
    QUERY_GET_RESET = 3


class SyslogNgCtlExecutor(ABC):
    @abstractmethod
    def run_command(
        self,
        instance_name: str,
        command_short_name: str,
        command: typing.List[str],
    ) -> typing.Dict[str, typing.Any]:
        pass

    @staticmethod
    def construct_ctl_stats_command(reset):
        stats_command = ["stats"]
        if reset:
            stats_command.append("--reset")
        return stats_command

    @staticmethod
    def construct_ctl_stats_prometheus_command():
        return ["stats", "prometheus"]

    @staticmethod
    def construct_ctl_credentials_command(credential, secret):
        return ["credentials", "add", credential, secret]

    @staticmethod
    def construct_ctl_query_command(pattern, query_type):
        query_command = ["query"]

        if query_type == QueryTypes.QUERY_GET:
            query_command += ["get"]
        elif query_type == QueryTypes.QUERY_SUM:
            query_command += ["get --sum"]
        elif query_type == QueryTypes.QUERY_LIST:
            query_command += ["list"]
        elif query_type == QueryTypes.QUERY_GET_RESET:
            query_command += ["get --reset"]

        query_command += [pattern]

        return query_command

    @staticmethod
    def construct_std_file_path(instance_name: str, command_short_name: str, std_type: str) -> Path:
        return Path("syslog_ng_ctl_{}_{}_{}".format(instance_name, command_short_name, std_type))



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import logging
import re

from axosyslog_light.common.file import File

logger = logging.getLogger(__name__)


class ConsoleLogReader(object):
    def __init__(self, instance_paths, teardown):

        self.console_log_files = {
            "stderr": {"path": instance_paths.get_stderr_path(), "file": File(instance_paths.get_stderr_path())},
            "stdout": {"path": instance_paths.get_stdout_path(), "file": File(instance_paths.get_stdout_path())},
            "syntax_only_stderr": {"path": instance_paths.get_syntax_only_stderr_path(), "file": File(instance_paths.get_syntax_only_stderr_path())},
            "syntax_only_stdout": {"path": instance_paths.get_syntax_only_stdout_path(), "file": File(instance_paths.get_syntax_only_stdout_path())},
        }
        self.__teardown = teardown

    def wait_for_start_message(self):
        syslog_ng_start_message = ["syslog-ng starting up;"]
        return self.wait_for_messages_in_console_log(syslog_ng_start_message)

    def wait_for_stop_message(self):
        syslog_ng_stop_message = ["syslog-ng shutting down"]
        return self.wait_for_messages_in_console_log(syslog_ng_stop_message)

    def wait_for_reload_message(self):
        syslog_ng_reload_messages = [
            "New configuration initialized",
            "Configuration reload request received, reloading configuration",
            "Configuration reload finished",
        ]
        return self.wait_for_messages_in_console_log(syslog_ng_reload_messages)

    def wait_for_message_in_console_log(self, expected_message):
        return self.wait_for_messages_in_console_log(self, [expected_message])

    def wait_for_messages_in_console_log(self, expected_messages):
        stderr_file = self.console_log_files["stderr"]["file"]
        if not stderr_file.is_opened():
            stderr_file.wait_for_creation()
            stderr_file.open("r")
            self.__teardown.register(stderr_file.close)
        return stderr_file.wait_for_lines(expected_messages)

    def check_for_unexpected_messages(self, unexpected_messages=None):
        unexpected_patterns = ["Plugin module not found", "assertion failed"]
        if unexpected_messages is not None:
            unexpected_patterns.extend(unexpected_messages)

        console_logs = self.__get_all_console_logs()
        for unexpected_pattern in unexpected_patterns:
            for console_log_message in console_logs.split("\n"):
                if re.search(unexpected_pattern, console_log_message) is not None:
                    logger.error("Found unexpected message in console log: {}".format(console_log_message))
                    raise Exception("Unexpected error log in console", console_log_message)

    def dump_stderr(self, last_n_lines=10):
        stderr = self.__get_all_console_logs()
        logger.error("\n".join(stderr.split("\n")[-last_n_lines:]))

    def __get_all_console_logs(self):
        console_logs = ""
        for log_type, log_path in self.console_log_files.items():
            if not log_path["path"].exists():
                logger.warning("Console log file {} does not exist".format(log_path))
                continue
            console_log_file = File(log_path["path"])
            console_log_file.open("r")
            console_logs += console_log_file.read()
            console_log_file.close()
        return console_logs

    @staticmethod
    def handle_valgrind_log(valgrind_log_path):
        with open(valgrind_log_path, "r") as valgrind_log:
            valgrind_content = valgrind_log.read()
            assert "Invalid read" not in valgrind_content
            assert "Invalid write" not in valgrind_content
            assert "blocks are definitely lost in loss record" not in valgrind_content
            assert "Uninitialised value was created by a heap allocation" not in valgrind_content



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import socket
import typing
from pathlib import Path


class UnixDgramIO():
    def __init__(self, path: Path) -> None:
        self.__socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        self.__path = path

    def write_messages(self, messages: typing.List[str]) -> None:
        for message in messages:
            self.__socket.sendto(message.encode(), str(self.__path))

    def close_socket(self):
        self.__socket.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
# Copyright (c) 2015-2019 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest
from axosyslog_light.testcase_parameters.testcase_parameters import TestcaseParameters


@pytest.fixture
def test_message():
    return "test message - öüóőúéáű"


@pytest.fixture
def fake_testcase_parameters(request, tmpdir):
    request.config.option.installdir = tmpdir.join("installdir")
    request.config.option.reports = tmpdir.join("reports")
    request.config.option.run_under = ""

    yield TestcaseParameters(request)


@pytest.fixture
def temp_file(tmpdir):
    return tmpdir.join("test_file.log")



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


class Rewrite(object):
    group_type = "rewrite"

    def __init__(self, driver_name, positional_parameters, **options):
        self.driver_name = driver_name
        self.options = options
        self.positional_parameters = positional_parameters


class SetTag(Rewrite):
    def __init__(self, tag, **options):
        super(SetTag, self).__init__("set_tag", [tag], **options)


class Set(Rewrite):
    def __init__(self, template, **options):
        super(Set, self).__init__("set", [template], **options)


class SetPri(Rewrite):
    def __init__(self, pri, **options):
        super(SetPri, self).__init__("set_pri", [pri], **options)


class CreditCardHash(Rewrite):
    def __init__(self, **options):
        super(CreditCardHash, self).__init__("credit-card-hash", [], **options)


class CreditCardMask(Rewrite):
    def __init__(self, **options):
        super(CreditCardMask, self).__init__("credit-card-mask", [], **options)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import logging

import psutil
from axosyslog_light.executors.command_executor import prepare_executable_command
from axosyslog_light.executors.command_executor import prepare_printable_command
from axosyslog_light.executors.command_executor import prepare_std_outputs

logger = logging.getLogger(__name__)


class ProcessExecutor(object):
    def __init__(self):
        self.process = None

    def start(self, command, stdout_path, stderr_path):
        printable_command = prepare_printable_command(command)
        executable_command = prepare_executable_command(command)
        stdout, stderr = prepare_std_outputs(stdout_path, stderr_path)
        env = psutil.Process().environ()
        if "VIRTUAL_ENV" in env:
            del env["VIRTUAL_ENV"]
        logger.info("Following process will be started:\n{}\n".format(printable_command))
        try:
            self.process = psutil.Popen(
                executable_command, stdout=stdout.open(mode="a"), stderr=stderr.open(mode="a"), env=env,
            )
        except (OSError, psutil.Error) as e:
            logger.error("Failed to start process: {}\nError: {}".format(printable_command, e))
            stdout.close()
            stderr.close()
        return self.process



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
# Copyright (c) 2020 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from __future__ import annotations

import typing
from abc import ABC
from abc import abstractmethod
from dataclasses import dataclass
from pathlib import Path

from psutil import Popen
from psutil import TimeoutExpired


@dataclass
class LoggenStartParams:
    target: str
    port: int
    client_port: int = 0
    inet: bool = False
    unix: bool = False
    stream: bool = False
    dgram: bool = False
    use_ssl: bool = False
    dont_parse: bool = False
    read_file: typing.Optional[Path] = None
    skip_tokens: typing.Optional[str] = None
    loop_reading: bool = False
    rate: typing.Optional[int] = None
    perf: bool = False
    rate_burst_start: bool = False
    interval: typing.Optional[int] = None
    permanent: bool = None
    syslog_proto: bool = None
    proxied: typing.Optional[int] = None
    sdata: bool = False
    no_framing: bool = False
    active_connections: typing.Optional[int] = None
    idle_connections: typing.Optional[int] = None
    ipv6: bool = False
    debug: bool = False
    number: typing.Optional[int] = None
    csv: bool = False
    quiet: bool = False
    size: typing.Optional[int] = None
    reconnect: bool = False
    proxied_tls_passthrough: bool = False
    proxy_src_ip: typing.Optional[str] = None
    proxy_dst_ip: typing.Optional[str] = None
    proxy_src_port: typing.Optional[int] = None
    proxy_dst_port: typing.Optional[int] = None

    def format(self) -> typing.List[str]:
        params = []
        if self.inet is True:
            params.append("--inet")
        if self.unix is True:
            params.append("--unix")
        if self.stream is True:
            params.append("--stream")
        if self.dgram is True:
            params.append("--dgram")
        if self.use_ssl is True:
            params.append("--use-ssl")
        if self.dont_parse is True:
            params.append("--dont-parse")
        if self.read_file is not None:
            params.append(f"--read-file={self.read_file}")
        if self.skip_tokens is not None:
            params.append(f"--skip-tokens={self.skip_tokens}")
        if self.loop_reading is True:
            params.append("--loop-reading")
        if self.rate is not None:
            params.append(f"--rate={self.rate}")
        if self.perf:
            params.append("--perf")
        if self.rate_burst_start:
            params.append("--rate-burst-start")
        if self.interval is not None:
            params.append(f"--interval={self.interval}")
        if self.permanent is True:
            params.append("--permanent")
        if self.syslog_proto is True:
            params.append("--syslog-proto")
        if self.proxied is True:
            params.append("--proxied")
        if self.proxied is not None:
            if self.proxied not in {1, 2}:
                raise ValueError(f"Proxied version must be 1 or 2, got {self.proxied}")
            params.append(f"--proxied={self.proxied}")
        if self.proxy_src_ip is not None:
            params.append(f"--proxy-src-ip={self.proxy_src_ip}")
        if self.proxy_dst_ip is not None:
            params.append(f"--proxy-dst-ip={self.proxy_dst_ip}")
        if self.proxy_src_port is not None:
            params.append(f"--proxy-src-port={self.proxy_src_port}")
        if self.proxy_dst_port is not None:
            params.append(f"--proxy-dst-port={self.proxy_dst_port}")
        if self.proxied_tls_passthrough is True:
            params.append("--proxied-tls-passthrough")
        if self.sdata is True:
            params.append("--sdata")
        if self.no_framing is True:
            params.append("--no-framing")
        if self.client_port:
            params.append(f"--client-port={self.client_port}")
        if self.active_connections is not None:
            params.append(f"--active-connections={self.active_connections}")
        if self.idle_connections is not None:
            params.append(f"--idle-connections={self.idle_connections}")
        if self.ipv6 is True:
            params.append("--ipv6")
        if self.debug is True:
            params.append("--debug")
        if self.number is not None:
            params.append(f"--number={self.number}")
        if self.csv is True:
            params.append("--csv")
        if self.quiet is True:
            params.append("--quiet")
        if self.size is not None:
            params.append(f"--size={self.size}")
        if self.reconnect is True:
            params.append("--reconnect")
        params.append(self.target)
        params.append(str(self.port))
        return params


class LoggenExecutor(ABC):
    DEFAULT_EXECUTOR: typing.Optional[LoggenExecutor] = None

    def __init__(self) -> None:
        self.proc: typing.Optional[Popen] = None

    @staticmethod
    def set_default_executor(loggen_executor: LoggenExecutor) -> None:
        LoggenExecutor.DEFAULT_EXECUTOR = loggen_executor.copy()

    @staticmethod
    def get_default_executor() -> typing.Optional[LoggenExecutor]:
        if LoggenExecutor.DEFAULT_EXECUTOR is None:
            return None
        return LoggenExecutor.DEFAULT_EXECUTOR.copy()

    def copy(self) -> LoggenExecutor:
        if self.proc is not None:
            raise Exception("You shouldn't copy a running executor")
        return self._copy()

    def start(self, start_params: LoggenStartParams, stderr: Path, stdout: Path, instance_index: int) -> Popen:
        if self.proc is not None and self.proc.is_running():
            raise Exception("Loggen is already running, you shouldn't call start")
        self.proc = self._start(start_params, stderr, stdout, instance_index)
        return self.proc

    def stop(self) -> None:
        if self.proc is None:
            return

        self.proc.terminate()
        try:
            self.proc.wait(4)
        except TimeoutExpired:
            self.proc.kill()

        self.proc = None

    @abstractmethod
    def _copy(self) -> LoggenExecutor:
        pass

    @abstractmethod
    def _start(self, start_params: LoggenStartParams, stderr: Path, stdout: Path) -> Popen:
        pass



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2019 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


def calculate_testcase_name(item_name):
    # In case of parametrized tests we need to replace "[" and "]" because
    # testcase name will appear in directory name
    return item_name.replace("[", "_").replace("]", "_").replace("/", "_")



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import functools

from axosyslog_light.common.operations import cast_to_list
from axosyslog_light.common.random_id import get_unique_id


class StatementGroup(list):
    def __init__(self, statements):
        super(StatementGroup, self).__init__(cast_to_list(statements))
        self.__group_type = self.__calculate_group_type(cast_to_list(statements))
        self.__group_id = "{}_{}".format(self.__group_type, get_unique_id())

    @staticmethod
    def __calculate_group_type(statements):
        def check_consistency(stmt1, stmt2):
            type1 = stmt1.group_type
            type2 = stmt2.group_type
            if type1 != type2:
                raise TypeError("Conflict in statement types: {} and {}".format(type1, type2))
            return stmt2

        return functools.reduce(check_consistency, statements).group_type

    @property
    def group_type(self):
        return self.__group_type

    @property
    def group_id(self):
        return self.__group_id



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import argparse
import logging
import os
import re
import uuid
from datetime import datetime
from pathlib import Path

import axosyslog_light.testcase_parameters.testcase_parameters as tc_parameters
import psutil
import pytest
import xdist
from axosyslog_light.common.file import copy_file
from axosyslog_light.common.pytest_operations import calculate_testcase_name
from axosyslog_light.common.session_data import get_session_data
from axosyslog_light.common.session_data import SessionData
from axosyslog_light.helpers.loggen.loggen import Loggen
from axosyslog_light.helpers.loggen.loggen_docker_executor import LoggenDockerExecutor
from axosyslog_light.helpers.loggen.loggen_executor import LoggenExecutor
from axosyslog_light.helpers.loggen.loggen_local_executor import LoggenLocalExecutor
from axosyslog_light.message_builder.bsd_format import BSDFormat
from axosyslog_light.message_builder.log_message import LogMessage
from axosyslog_light.syslog_ng.syslog_ng import SyslogNg
from axosyslog_light.syslog_ng.syslog_ng_docker_executor import SyslogNgDockerExecutor
from axosyslog_light.syslog_ng.syslog_ng_local_executor import SyslogNgLocalExecutor
from axosyslog_light.syslog_ng.syslog_ng_paths import SyslogNgPaths
from axosyslog_light.syslog_ng_config.syslog_ng_config import SyslogNgConfig
from axosyslog_light.syslog_ng_ctl.syslog_ng_ctl import SyslogNgCtl
from axosyslog_light.syslog_ng_ctl.syslog_ng_ctl_docker_executor import SyslogNgCtlDockerExecutor
from axosyslog_light.syslog_ng_ctl.syslog_ng_ctl_local_executor import SyslogNgCtlLocalExecutor
from axosyslog_light.testcase_parameters.testcase_parameters import TestcaseParameters

logger = logging.getLogger(__name__)


class InstallDirAction(argparse.Action):
    def __call__(self, parser, namespace, path, option_string=None):
        install_dir = Path(path)

        if not install_dir.is_dir():
            raise argparse.ArgumentTypeError("{0} is not a valid directory".format(path))

        binary = Path(install_dir, "sbin/syslog-ng")
        if not binary.exists():
            raise argparse.ArgumentTypeError("{0} not exist".format(binary))

        setattr(namespace, self.dest, path)


# Command line options
def pytest_addoption(parser):
    parser.addoption("--runslow", action="store_true", default=False, help="run slow tests")
    parser.addoption("--run-under", help="Run syslog-ng under selected tool, example tools: [valgrind, strace, gdb, gdb_for_bt]")

    parser.addoption(
        "--runner",
        default="local",
        choices=["local", "docker"],
        help="How to run AxoSyslog.",
    )
    parser.addoption(
        "--installdir",
        action=InstallDirAction,
        help="Look for AxoSyslog binaries here. Used when 'runner' is 'local'. Example path: '/home/user/axosyslog/install/'",
    )
    parser.addoption(
        "--docker-image",
        default="ghcr.io/axoflow/axosyslog:latest",
        help="Docker image to use for running syslog-ng. Used when 'runner' is 'docker'. Default: ghcr.io/axoflow/axosyslog:latest",
    )

    parser.addoption(
        "--reports",
        action="store",
        help="Path for report files folder. Default form: 'reports/<current_date>'",
    )


def get_current_date():
    return datetime.now().strftime("%Y-%m-%d-%H-%M-%S-%f")


# Pytest Hooks
def pytest_collection_modifyitems(config, items):
    if config.getoption("--runslow"):
        return
    skip_slow = pytest.mark.skip(reason="need --runslow option to run")
    for item in items:
        if "slow" in item.keywords:
            item.add_marker(skip_slow)


def pytest_runtest_logreport(report):
    if report.outcome == "failed":
        logger.error("\n{}".format(report.longrepr))


# Pytest Fixtures
@pytest.fixture
def testcase_parameters(request):
    parameters = TestcaseParameters(request)
    tc_parameters.INSTANCE_PATH = SyslogNgPaths(parameters).set_syslog_ng_paths("server")
    return parameters


@pytest.fixture
def config(request, syslog_ng, teardown) -> SyslogNgConfig:
    return syslog_ng.create_config(request.getfixturevalue("config_version"), teardown)


@pytest.fixture
def syslog_ng(request: pytest.FixtureRequest, testcase_parameters: TestcaseParameters, syslog_ng_ctl: SyslogNgCtl, container_name: str, teardown):
    if request.config.getoption("--runner") == "local":
        executor = SyslogNgLocalExecutor(tc_parameters.INSTANCE_PATH.get_syslog_ng_bin(), testcase_parameters)
    elif request.config.getoption("--runner") == "docker":
        executor = SyslogNgDockerExecutor(container_name, request.config.getoption("--docker-image"))
    else:
        raise ValueError("Invalid runner")

    syslog_ng = SyslogNg(executor, syslog_ng_ctl, tc_parameters.INSTANCE_PATH, testcase_parameters, teardown)
    teardown.register(syslog_ng.stop)
    return syslog_ng


class TeardownRegistry:
    teardown_callbacks = []

    def register(self, teardown_callback):
        TeardownRegistry.teardown_callbacks.append(teardown_callback)

    def execute_teardown_callbacks(self):
        for teardown_callback in TeardownRegistry.teardown_callbacks:
            teardown_callback()


@pytest.fixture(scope="function")
def teardown():
    teardown_registry = TeardownRegistry()
    yield teardown_registry
    teardown_registry.execute_teardown_callbacks()


@pytest.fixture
def syslog_ng_ctl(request: pytest.FixtureRequest, testcase_parameters, container_name):
    if request.config.getoption("--runner") == "local":
        executor = SyslogNgCtlLocalExecutor(
            tc_parameters.INSTANCE_PATH.get_syslog_ng_ctl_bin(),
            tc_parameters.INSTANCE_PATH.get_control_socket_path(),
        )
    elif request.config.getoption("--runner") == "docker":
        executor = SyslogNgCtlDockerExecutor(container_name)
    else:
        raise ValueError("Invalid runner")

    return SyslogNgCtl(tc_parameters.INSTANCE_PATH, executor)


@pytest.fixture
def container_name(request: pytest.FixtureRequest, testcase_parameters: TestcaseParameters):
    container_name = f"{testcase_parameters.get_testcase_name()}_{tc_parameters.INSTANCE_PATH.get_instance_name()}"
    return re.sub(r'[^a-zA-Z0-9_.-]', '_', container_name)


@pytest.fixture
def bsd_formatter():
    return BSDFormat()


@pytest.fixture
def log_message():
    return LogMessage()


@pytest.fixture
def version(syslog_ng):
    return syslog_ng.get_version()


@pytest.fixture
def config_version(syslog_ng):
    return syslog_ng.get_config_version()


@pytest.fixture
def loggen(testcase_parameters):
    server = Loggen()
    yield server
    server.stop()


def calculate_report_file_path(working_dir):
    return Path(working_dir, "testcase.reportlog")


def chdir_to_light_base_dir():
    absolute_light_base_dir = Path(__file__).parents[2]
    os.chdir(absolute_light_base_dir)


def __calculate_testcase_dir(name):
    with get_session_data() as session_data:
        return Path(session_data["reports_dir"], calculate_testcase_name(name))


@pytest.fixture
def testcase_dir(request):
    return __calculate_testcase_dir(request.node.name)


def pytest_runtest_setup(item):
    logging_plugin = item.config.pluginmanager.get_plugin("logging-plugin")
    testcase_dir = __calculate_testcase_dir(item.name)
    logging_plugin.set_log_path(calculate_report_file_path(testcase_dir))
    os.chdir(testcase_dir)


def pytest_sessionstart(session):
    if xdist.is_xdist_controller(session):
        return

    base_number_of_open_fds = len(psutil.Process().open_files())

    with get_session_data() as session_data:
        testrunuid = os.environ.get("PYTEST_XDIST_TESTRUNUID", uuid.uuid4().hex)  # generate one if not running in xdist
        if session_data.get("testrunuid") != testrunuid:
            session_data.clear()
            session_data["testrunuid"] = testrunuid

        session_data["active_workers"] = session_data.get("active_workers", 0) + 1

        if session_data.get("session_started", False):
            return

        try:
            reports_dir = Path(session.config.getoption("--reports")).resolve().absolute()
        except TypeError:
            reports_dir = Path("reports", get_current_date()).resolve().absolute()

        reports_dir.mkdir(parents=True, exist_ok=True)
        if reports_dir.parent.name == "reports":
            last_reports_dir = Path(reports_dir.parent, "last")
            last_reports_dir.unlink(True)
            last_reports_dir.symlink_to(reports_dir)

        session_data["session_started"] = True
        session_data["reports_dir"] = reports_dir
        session_data["base_number_of_open_fds"] = base_number_of_open_fds


def pytest_sessionfinish(session, exitstatus):
    if xdist.is_xdist_controller(session):
        return

    with get_session_data() as session_data:
        active_workers = session_data["active_workers"] = session_data["active_workers"] - 1

    if active_workers == 0:
        SessionData.cleanup()


def light_extra_files(target_dir):
    if "LIGHT_EXTRA_FILES" in os.environ:
        for f in os.environ["LIGHT_EXTRA_FILES"].split(":"):
            if Path(f).exists():
                copy_file(f, target_dir)


@pytest.fixture(autouse=True)
def setup(request):
    with get_session_data() as session_data:
        base_number_of_open_fds = session_data["base_number_of_open_fds"]
    number_of_open_fds = len(psutil.Process().open_files())
    assert base_number_of_open_fds + 1 == number_of_open_fds, "Previous testcase has unclosed opened fds"
    assert len(psutil.Process().net_connections(kind="inet")) == 0, "Previous testcase has unclosed opened sockets"
    testcase_parameters = request.getfixturevalue("testcase_parameters")

    copy_file(testcase_parameters.get_testcase_file(), Path.cwd())
    light_extra_files(Path.cwd())
    request.addfinalizer(lambda: logger.info("Report file path\n{}\n".format(calculate_report_file_path(Path.cwd()))))

    if request.config.getoption("--runner") == "docker":
        LoggenExecutor.set_default_executor(
            LoggenDockerExecutor(request.config.getoption("--docker-image")),
        )
    elif request.config.getoption("--runner") == "local":
        LoggenExecutor.set_default_executor(
            LoggenLocalExecutor(Path(request.config.getoption("--installdir"), "bin", "loggen")),
        )


@pytest.fixture
def port_allocator():
    def get_next_port() -> int:
        with get_session_data() as session_data:
            last_port = session_data.get("port_allocator_last_port", 20000)
            port = last_port + 1
            session_data["port_allocator_last_port"] = port
        return port

    return get_next_port



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import os
import typing
from copy import copy
from pathlib import Path
from subprocess import Popen

from axosyslog_light.executors.process_executor import ProcessExecutor
from axosyslog_light.syslog_ng.syslog_ng_executor import SyslogNgExecutor
from axosyslog_light.syslog_ng.syslog_ng_executor import SyslogNgStartParams


class SyslogNgDockerExecutor(SyslogNgExecutor):
    def __init__(self, container_name: str, image_name: str, extra_volume_mounts: typing.Set[typing.Tuple[str, str]] = None, extra_env_vars: typing.Optional[dict] = None) -> None:
        self.__process_executor = ProcessExecutor()
        self.__container_name = container_name
        self.__image_name = image_name
        self.__extra_volume_mounts = extra_volume_mounts
        self.__extra_env_vars = extra_env_vars

    def run_process(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
    ) -> Popen:
        start_params = copy(start_params)
        start_params.control_socket_path = Path("/tmp/syslog-ng.ctl")

        paths = {
            Path.cwd().absolute(),
            Path(start_params.config_path).parent.absolute(),
            Path(start_params.persist_path).parent.absolute(),
            Path(start_params.pid_path).parent.absolute(),
        }

        command = ["docker", "run", "--rm", "-i"]
        command += ["--name", self.__container_name]
        command += ["--workdir", str(Path.cwd().absolute())]
        command += ["--user", f"{os.getuid()}:{os.getgid()}"]
        command += ["-e", f"PUID={os.getuid()}", "-e", f"PGID={os.getgid()}"]
        command += ["--network", "host"]

        for path in paths:
            command += ["-v", f"{path}:{path}"]

        if self.__extra_env_vars:
            for key, value in self.__extra_env_vars.items():
                command += ["-e", f"{key}={value}"]

        if self.__extra_volume_mounts:
            for host_path, container_path in self.__extra_volume_mounts:
                command += ["-v", f"{host_path}:{container_path}"]

        command += [self.__image_name]
        command += start_params.format()

        return self.__process_executor.start(
            command=command,
            stdout_path=stdout_path,
            stderr_path=stderr_path,
        )

    def run_process_with_valgrind(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
        valgrind_output_path: Path,
    ) -> Popen:
        raise NotImplementedError()

    def run_process_with_gdb(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
    ) -> Popen:
        raise NotImplementedError()

    def run_process_with_strace(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
        strace_output_path: Path,
    ) -> Popen:
        raise NotImplementedError()

    def get_backtrace_from_core(
        self,
        core_file_path: Path,
        stderr_path: Path,
        stdout_path: Path,
    ) -> typing.Dict[str, typing.Any]:
        raise NotImplementedError()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
# flake8: noqa: F401, F811
from axosyslog_light.common.blocking import wait_until_false_custom
from axosyslog_light.common.blocking import wait_until_true_custom


def inner_function_return_true():
    return True


def inner_function_return_false():
    return False


def inner_function_add_numbers():
    return 2 + 3


def test_wait_until_true_inner_function_returns_true():
    assert wait_until_true_custom(inner_function_return_true, timeout=0.1)


def test_wait_until_true_custom_inner_function_returns_false():
    assert wait_until_true_custom(inner_function_return_false, timeout=0.1) is False


def test_wait_until_false_inner_function_returns_true():
    assert wait_until_false_custom(inner_function_return_true, timeout=0.1) is False


def test_wait_until_false_inner_function_returns_false():
    assert wait_until_false_custom(inner_function_return_false, timeout=0.1)


def test_wait_until_true_custom_returns_with_result():
    assert wait_until_true_custom(inner_function_add_numbers, timeout=0.1) == 5



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import logging
from pathlib import Path

from axosyslog_light.driver_io.file.file_io import FileIO
from axosyslog_light.syslog_ng_config.statements.sources.source_driver import SourceDriver
from axosyslog_light.syslog_ng_ctl.legacy_stats_handler import LegacyStatsHandler
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler

logger = logging.getLogger(__name__)


class FileSource(SourceDriver):
    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        file_name,
        **options,
    ) -> None:
        self.driver_name = "file"
        self.set_path(file_name)
        self.io = FileIO(self.get_path())
        super(FileSource, self).__init__(stats_handler, prometheus_stats_handler, [self.path], options)

    def get_path(self):
        return self.path

    def set_path(self, pathname):
        self.path = Path(pathname)

    def write_log(self, log, counter=1):
        for _ in range(counter):
            self.io.write_message(log)
        logger.info(
            "Content has been written to\nresource: {}\n"
            "number of times: {}\n"
            "content: {}\n".format(self.get_path(), counter, log),
        )

    def write_logs(self, logs, counter=1):
        for _ in range(counter):
            self.io.write_messages(logs)
        logger.info(
            "Content has been written to\nresource: {}\n"
            "number of times: {}\n"
            "content: {}\n".format(self.get_path(), counter, logs),
        )

    def close_file(self):
        self.io.close_writeable_file()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import typing

from axosyslog_light.driver_io.http.http_io import HttpIO
from axosyslog_light.syslog_ng_config.statements.sources.source_driver import SourceDriver
from axosyslog_light.syslog_ng_ctl.legacy_stats_handler import LegacyStatsHandler
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler


class WebhookSource(SourceDriver):
    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        **options,
    ) -> None:
        self.io = HttpIO(port=options.get("port"), use_tls="tls_key_file" in options)

        self.driver_name = "webhook"
        super(WebhookSource, self).__init__(stats_handler, prometheus_stats_handler, options=options)

    def write_log(self, message: str) -> None:
        self.io.write_message(message)

    def write_logs(self, messages: typing.List[str]) -> None:
        for message in messages:
            self.io.write_message(message)


class WebhookJsonSource(SourceDriver):
    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        **options,
    ) -> None:
        self.io = HttpIO(port=options.get("port"), use_tls="tls_key_file" in options)

        self.driver_name = "webhook-json"
        super(WebhookJsonSource, self).__init__(stats_handler, prometheus_stats_handler, options=options)

    def write_log(self, message: str) -> None:
        self.io.write_json_message(message)

    def write_logs(self, messages: typing.List[str]) -> None:
        for message in messages:
            self.io.write_json_message(message)



#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import asyncio
import logging
import socket
from abc import ABC
from abc import abstractmethod

from axosyslog_light.common.asynchronous import BackgroundEventLoop


logger = logging.getLogger(__name__)


class SingleConnectionStreamServer(ABC):
    def __init__(self):
        self._event_loop = BackgroundEventLoop()
        self._server = None
        self._client = self._event_loop.run_async(self._create_client()).result()

    async def start(self):
        server_res = await self._start_server()
        self._server = server_res

    async def stop(self):
        logger.debug("Shutting down SingleConnectionStreamServer")
        await self.close_client()
        await self._stop_server()

    async def close_client(self):
        """After a client connection is accepted, new connections will be rejected until this method is called"""
        await self._client.close()

    async def _create_client(self):
        return self.Client()

    @abstractmethod
    async def _start_server(self):
        pass

    async def _stop_server(self):
        if self._server is not None:
            self._server.close()
            await self._server.wait_closed()

    async def _client_accepted_cb(self, reader, writer):
        self._client.accept_connection(reader, writer)

    async def readexactly(self, n):
        await self._client.wait_for_client_connection()
        return await self._client._reader.readexactly(n)

    async def readuntil(self, separator=b'\n'):
        await self._client.wait_for_client_connection()
        return await self._client._reader.readuntil(separator)

    async def readline(self):
        await self._client.wait_for_client_connection()
        return await self._client._reader.readline()

    async def write(self, data):
        await self._client.wait_for_client_connection()
        self._client._writer.write(data)
        await self._client._writer.drain()

    class Client(object):
        def __init__(self):
            self._connection_accepted = asyncio.Event()
            self._reader = None
            self._writer = None

        def accept_connection(self, reader, writer):
            if self._connection_accepted.is_set():
                writer.close()
                return
            self._reader = reader
            self._writer = writer
            self._connection_accepted.set()

        async def wait_for_client_connection(self):
            await self._connection_accepted.wait()

        async def close(self):
            if not self._connection_accepted.is_set():
                return
            self._writer.close()
            try:
                await self._writer.wait_closed()
            except BrokenPipeError as e:

                # In case of TLS, we might want to send a connection close
                # alert to the peer, which may have already gone away.  In
                # that case we would receive an EPIPE, which is best
                # ignored.

                logger.warning("SingleConnectionStreamServer, ignoring BrokenPipeError in Client.close(): {}".format(e))
            self._connection_accepted.clear()


class SingleConnectionTCPServer(SingleConnectionStreamServer):
    def __init__(self, port, host=None, ip_protocol_version=socket.AF_INET, ssl=None):
        super(SingleConnectionTCPServer, self).__init__()
        self._host = host
        self._port = port
        self._family = ip_protocol_version
        self._ssl = ssl

    async def _start_server(self):
        logger.info("SingleConnectionTCPServer has been started on host:{}, on port:{}".format(self._host, self._port))
        server = await asyncio.start_server(self._client_accepted_cb, self._host, self._port, family=self._family, ssl=self._ssl)
        await server.start_serving()
        return server


class SingleConnectionUnixStreamServer(SingleConnectionStreamServer):
    def __init__(self, path, ssl=None):
        super(SingleConnectionUnixStreamServer, self).__init__()
        self._path = path
        self._ssl = ssl

    async def _start_server(self):
        logger.info("SingleConnectionUnixStreamServer has been started on path:{}".format(self._path))
        server = await asyncio.start_unix_server(self._client_accepted_cb, self._path, ssl=self._ssl)
        await server.start_serving()
        return server


class DatagramServer(ABC):
    def __init__(self):
        self._event_loop = BackgroundEventLoop()
        self._sock = None

    @abstractmethod
    async def start(self):
        pass

    async def stop(self):
        logger.debug("Shutting down DatagramServer")
        if self._sock is not None:
            self._sock.close()
            self._sock = None

    async def read_dgram(self, maxsize=65536):
        return await self._event_loop.loop.sock_recv(self._sock, maxsize)

    async def write_dgram(self, data):
        return await self._event_loop.loop.sock_sendall(self._sock, data)


class UDPServer(DatagramServer):
    def __init__(self, port, host=None, ip_protocol_version=socket.AF_INET):
        super(UDPServer, self).__init__()
        self._host = '' if host is None else host
        self._port = port
        self._family = ip_protocol_version

    async def start(self):
        logger.info("UDPServer has been started on host:{}, on port:{}".format(self._host, self._port))
        self._sock = socket.socket(self._family, socket.SOCK_DGRAM)
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._sock.setblocking(False)
        self._sock.bind((self._host, self._port))


class UnixDatagramServer(DatagramServer):
    def __init__(self, path):
        super(UnixDatagramServer, self).__init__()
        self._path = path

    async def start(self):
        logger.info("UnixDatagramServer has been started on path:{}".format(self._path))
        self._sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        self._sock.setblocking(False)
        self._sock.bind(str(self._path))



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import socket
import ssl
from enum import Enum
from enum import IntEnum
from pathlib import Path

from axosyslog_light.common.asynchronous import BackgroundEventLoop
from axosyslog_light.common.blocking import DEFAULT_TIMEOUT
from axosyslog_light.common.file import File
from axosyslog_light.common.file import get_shared_file
from axosyslog_light.common.network import SingleConnectionTCPServer
from axosyslog_light.common.network import UDPServer
from axosyslog_light.common.random_id import get_unique_id
from axosyslog_light.driver_io import message_readers
from axosyslog_light.helpers.loggen.loggen import Loggen
from axosyslog_light.helpers.loggen.loggen import LoggenStartParams


class NetworkIO():
    def __init__(self, ip, port, transport, framed, ip_proto_version=None):
        self.__ip = ip
        self.__port = port
        self.__transport = transport
        self.__framed = framed
        self.__ip_proto_version = NetworkIO.IPProtoVersion.V4 if ip_proto_version is None else ip_proto_version
        self.__server = None
        self.__message_reader = None

    def write_raw(self, raw_content, rate=None, transport=None, extra_loggen_args=None):
        if transport is None:
            transport = self.__transport

        if extra_loggen_args is None:
            extra_loggen_args = {}

        loggen_input_file_path = Path("loggen_input_{}.txt".format(get_unique_id()))

        loggen_input_file = File(loggen_input_file_path)
        loggen_input_file.write_content_and_close(raw_content)

        Loggen().start(
            LoggenStartParams(
                target=self.__ip,
                port=self.__port,
                read_file=str(loggen_input_file_path),
                dont_parse=True,
                permanent=True,
                rate=rate,
                **transport.value,
                **extra_loggen_args,
            ),
        )

    def __format_messages(self, messages, framed=None):
        if framed is None:
            framed = self.__framed
        if framed:
            return "".join([str(len(message.encode(encoding="utf-8"))) + " " + message for message in messages])
        return "".join([message + "\n" for message in messages])

    def write_messages(self, messages, rate=None, transport=None, framed=None, rate_burst_start=False, client_port=0):
        self.write_raw(
            self.__format_messages(messages, framed=framed), rate=rate, transport=transport,
            extra_loggen_args={"rate_burst_start": rate_burst_start, "client_port": client_port},
        )

    def write_messages_with_proxy_header(self, proxy_version, src_ip, dst_ip, src_port, dst_port, messages, rate=None, transport=None, framed=None, client_port=0):
        self.write_raw(
            self.__format_messages(messages, framed=framed),
            rate=rate,
            transport=transport,
            extra_loggen_args={
                "proxied": proxy_version,
                "proxy_src_ip": src_ip,
                "proxy_dst_ip": dst_ip,
                "proxy_src_port": str(src_port),
                "proxy_dst_port": str(dst_port),
                "client_port": client_port,
            },
        )

    def start_listener(self):
        self.__message_reader, self.__server = self.__transport.construct(self.__framed, self.__port, self.__ip, self.__ip_proto_version)
        BackgroundEventLoop().wait_async_result(self.__server.start(), timeout=DEFAULT_TIMEOUT)

    def stop_listener(self):
        if self.__message_reader is not None:
            BackgroundEventLoop().wait_async_result(self.__server.stop(), timeout=DEFAULT_TIMEOUT)
            self.__message_reader = None
            self.__server = None

    def read_number_of_messages(self, counter, timeout=DEFAULT_TIMEOUT):
        return self.__message_reader.wait_for_number_of_messages(counter, timeout)

    def read_until_messages(self, lines, timeout=DEFAULT_TIMEOUT):
        return self.__message_reader.wait_for_messages(lines, timeout)

    class IPProtoVersion(IntEnum):
        V4 = socket.AF_INET
        V6 = socket.AF_INET6

    class Transport(Enum):
        AUTO = {}
        TCP = {"inet": True, "stream": True}
        UDP = {"dgram": True}
        TLS = {"use_ssl": True}
        PROXIED_TCP = {"inet": True, "stream": True}
        PROXIED_TLS = {"use_ssl": True}
        PROXIED_TLS_PASSTHROUGH = {"use_ssl": True, "proxied_tls_passthrough": True}

        def construct(self, framed, port, host=None, ip_proto_version=None):
            def _construct(server, reader_class):
                return reader_class(server), server

            tls = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
            tls.load_cert_chain(get_shared_file("valid-localhost.crt"), get_shared_file("valid-localhost.key"))

            transport_mapping = {
                NetworkIO.Transport.TCP: lambda: _construct(SingleConnectionTCPServer(port, host, ip_proto_version), None if framed else message_readers.SingleLineStreamReader),
                NetworkIO.Transport.TLS: lambda: _construct(SingleConnectionTCPServer(port, host, ip_proto_version, tls), None if framed else message_readers.SingleLineStreamReader),
                NetworkIO.Transport.UDP: lambda: _construct(UDPServer(port, host, ip_proto_version), message_readers.DatagramReader),
            }
            return transport_mapping[self]()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balázs Scheidler <balazs.scheidler@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from axosyslog_light.common.random_id import get_unique_id


class Template(object):
    def __init__(self, template, name=None, escape=None, use_simple_statement=False):
        self.template = template
        self.template_escape = escape
        self.name = name or "template_{}".format(get_unique_id())
        self.use_simple_statement = escape is not None or use_simple_statement


class TemplateFunction(object):
    def __init__(self, template, name):
        self.template = template
        self.name = name



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import re
import typing

from axosyslog_light.syslog_ng_ctl.syslog_ng_ctl import SyslogNgCtl


class LegacyStatsHandler(object):
    def __init__(self, syslog_ng_ctl: SyslogNgCtl) -> None:
        self.__syslog_ng_ctl = syslog_ng_ctl

    def get_query(self, group_type: str, driver_name: str) -> typing.Dict[str, int]:
        component = LegacyStatsHandler.__build_component(group_type, driver_name)
        query_pattern = LegacyStatsHandler.__build_query_pattern(component)
        query_result = self.__syslog_ng_ctl.query(pattern="*{}*".format(query_pattern))['stdout'].splitlines()

        return self.__parse_result(result=query_result, result_type="query")

    def get_stats(self, group_type: str, driver_name: str, driver_instance: str = "") -> typing.Dict[str, int]:
        component = LegacyStatsHandler.__build_component(group_type, driver_name)
        stats_pattern = LegacyStatsHandler.__build_stats_pattern(component)
        stats_result = self.__syslog_ng_ctl.stats()['stdout'].splitlines()

        filtered_stats_result = LegacyStatsHandler.__filter_stats_result(stats_result, stats_pattern, driver_instance)
        return LegacyStatsHandler.__parse_result(result=filtered_stats_result, result_type="stats")

    @staticmethod
    def __build_component(group_type: str, driver_name: str) -> str:
        if group_type == "destination":
            statement_short_name = "dst"
            return "{}.{}".format(statement_short_name, driver_name)
        elif group_type == "source":
            statement_short_name = "src"
            return "{}.{}".format(statement_short_name, driver_name)
        elif group_type == "parser":
            return "{}".format(group_type)
        elif group_type == "filter":
            return "{}".format(group_type)
        else:
            raise Exception("Unknown group_type: {}".format(group_type))

    @staticmethod
    def __build_query_pattern(component: str) -> str:
        return LegacyStatsHandler.__build_pattern(component=component, delimiter=".")

    @staticmethod
    def __build_stats_pattern(component: str) -> str:
        return LegacyStatsHandler.__build_pattern(component=component, delimiter=";")

    @staticmethod
    def __build_pattern(component: str, delimiter: str) -> str:
        return f"{component}{delimiter}"

    @staticmethod
    def __parse_query_result(line: str) -> typing.Dict[str, int]:
        split = re.split(r"\.|=", line)
        if len(split) < 2:
            return dict()

        key, value = split[-2:]
        return {key: int(value)}

    @staticmethod
    def __parse_stats_result(line: str) -> typing.Dict[str, int]:
        split = line.split(";")
        if len(split) < 2:
            return dict()

        key, value = split[-2:]
        return {key: int(value)}

    @staticmethod
    def __parse_result(result: typing.List[str], result_type: str) -> typing.Dict[str, int]:
        parsed_output = {}
        for line in result:
            if result_type == "query":
                parsed_output.update(LegacyStatsHandler.__parse_query_result(line))
            elif result_type == "stats":
                parsed_output.update(LegacyStatsHandler.__parse_stats_result(line))
            else:
                raise Exception("Unknown result_type: {}".format(result_type))
        return parsed_output

    @staticmethod
    def __filter_stats_result(
        stats_result: typing.List[str],
        stats_pattern: str,
        driver_instance: str,
    ) -> typing.List[str]:
        filtered_list = []
        for stats_line in stats_result:
            if stats_pattern in stats_line and driver_instance in stats_line:
                filtered_list.append(stats_line)
        return filtered_list



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import typing

import requests


class HttpIO():
    def __init__(self, port: typing.Optional[int] = None, use_tls: bool = False) -> None:
        if port is None:
            port = 443 if use_tls else 80
        self.__url = f"{'https' if use_tls else 'http'}://localhost:{port}/"

    def write_message(self, message: str) -> None:
        response = requests.post(self.__url, data=message)
        if response.status_code != 200:
            raise Exception(f"Failed to send message: {response.status_code} {response.text}")

    def write_json_message(self, message: dict) -> None:
        response = requests.post(self.__url, json=message)
        if response.status_code != 200:
            raise Exception(f"Failed to send message: {response.status_code} {response.text}")



#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import asyncio
import concurrent
import threading


class BackgroundEventLoop(object):
    """Singleton event loop running in the background"""
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super(BackgroundEventLoop, cls).__new__(cls)
            cls._instance._init()
        return cls._instance

    def _init(self):
        self.loop = asyncio.new_event_loop()
        self.thread = threading.Thread(target=self._run_event_loop_func, daemon=True)
        self.thread.start()

    def _run_event_loop_func(self):
        asyncio.set_event_loop(self.loop)
        self.loop.run_forever()

    def run_async(self, async_func):
        return asyncio.run_coroutine_threadsafe(async_func, self.loop)

    def wait_async_result(self, async_func, timeout=None):
        future = self.run_async(async_func)
        try:
            return future.result(timeout=timeout)
        except (asyncio.TimeoutError, TimeoutError, concurrent.futures.TimeoutError) as err:
            future.cancel()
            raise asyncio.TimeoutError("Timeout expired for background event. async_func={}, timeout={} sec".format(async_func, timeout)) from err

    def stop(self):
        self.loop.call_soon_threadsafe(self.loop.stop)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import os
import re
from pathlib import Path

import pytest
from axosyslog_light.common.blocking import wait_until_true
from axosyslog_light.common.file import File
from axosyslog_light.executors.process_executor import ProcessExecutor
from psutil import TimeoutExpired


class SNMPtrapd(object):
    TRAP_LOG_PREFIX = 'LIGHT_TEST_SNMP_TRAP_RECEIVED:'

    def __init__(self, port):
        self.snmptrapd_proc = None
        self.port = port

        self.snmptrapd_log = Path("snmptrapd_log")
        self.snmptrapd_stdout_path = Path("snmptrapd_stdout")
        self.snmptrapd_stderr_path = Path("snmptrapd_stderr")

    def wait_for_snmptrapd_log_creation(self):
        return self.snmptrapd_log.exists()

    def wait_for_snmptrapd_startup(self):
        return "NET-SNMP version" in self.snmptrapd_log.read_text()

    def start(self):
        if self.snmptrapd_proc is not None:
            return

        self.snmptrapd_proc = ProcessExecutor().start(
            [
                "snmptrapd", "-f",
                "--disableAuthorization=yes",
                "-C",
                "-m ALL",
                "-A",
                "-Ddump",
                "-On",
                "--doNotLogTraps=no",
                "--authCommunity=log public",
                self.port,
                "-d",
                "-Lf", os.path.relpath(str(self.snmptrapd_log)),
                "-F", "{}%v\n".format(self.TRAP_LOG_PREFIX),
            ],
            self.snmptrapd_stdout_path,
            self.snmptrapd_stderr_path,
        )
        wait_until_true(self.wait_for_snmptrapd_log_creation)
        wait_until_true(self.wait_for_snmptrapd_startup)
        return self.snmptrapd_proc.is_running()

    def stop(self):
        if self.snmptrapd_proc is None:
            return

        self.snmptrapd_proc.terminate()
        try:
            self.snmptrapd_proc.wait(4)
        except TimeoutExpired:
            self.snmptrapd_proc.kill()

        self.snmptrapd_proc = None

    def get_port(self):
        return self.port

    def get_traps(self, counter):
        trap_list = []

        f = File(self.snmptrapd_log)
        f.open("r")

        while True:
            trap_line = f.wait_for_lines([self.TRAP_LOG_PREFIX])[0]
            res = re.match('({})(.*)'.format(self.TRAP_LOG_PREFIX), trap_line)
            if (res):
                trap_list.extend(res.group(2).rstrip().split("\t"))
            if len(trap_list) == counter:
                break

        f.close()

        return sorted(trap_list)

    def get_log(self):
        f = File(self.snmptrapd_log)
        f.open("r")

        log = f.read()

        f.close()

        return log


@pytest.fixture
def snmptrapd(port_allocator):
    server = SNMPtrapd(port_allocator())
    server.start()
    yield server
    server.stop()


class SNMPTestParams(object):
    def __init__(self):
        pass

    def get_ip_address(self):
        return '"127.0.0.1"'

    def get_default_community(self):
        return 'public'

    def get_basic_snmp_obj(self):
        return '".1.3.6.1.4.1.18372.3.1.1.1.1.1.0", "Octetstring", "admin"'

    def get_basic_trap_obj(self):
        return '".1.3.6.1.6.3.1.1.4.1.0", "Objectid", ".1.3.6.1.4.1.18372.3.1.1.1.2.1"'

    def get_cisco_trap_obj(self):
        return '".1.3.6.1.6.3.1.1.4.1.0","Objectid",".1.3.6.1.4.1.9.9.41.2.0.1"'

    def get_cisco_snmp_obj(self):
        cisco_snmp_obj = (
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.2.55", "Octetstring", "SYS"',
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.3.55", "Integer", "6"',
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.4.55", "Octetstring", "CONFIG_I"',
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.5.55", "Octetstring", "Configured from console by vty1 (10.30.0.32)"',
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.6.55", "Timeticks", "97881"',
        )
        return cisco_snmp_obj

    def get_expected_cisco_trap(self):
        return sorted([
            '.1.3.6.1.4.1.9.9.41.1.2.3.1.2.55 = STRING: "SYS"',
            '.1.3.6.1.4.1.9.9.41.1.2.3.1.3.55 = INTEGER: 6',
            '.1.3.6.1.4.1.9.9.41.1.2.3.1.4.55 = STRING: "CONFIG_I"',
            '.1.3.6.1.4.1.9.9.41.1.2.3.1.5.55 = STRING: "Configured from console by vty1 (10.30.0.32)"',
            '.1.3.6.1.4.1.9.9.41.1.2.3.1.6.55 = Timeticks: (97881) 0:16:18.81',
            '.1.3.6.1.6.3.1.1.4.1.0 = OID: .1.3.6.1.4.1.18372.3.1.1.1.2.1',
        ])

    def get_expected_basic_trap(self):
        return sorted([
            '.1.3.6.1.4.1.18372.3.1.1.1.1.1.0 = STRING: "admin"',
            '.1.3.6.1.6.3.1.1.4.1.0 = OID: .1.3.6.1.4.1.18372.3.1.1.1.2.1',
        ])

    def get_expected_empty_trap(self):
        return [
            '.1.3.6.1.6.3.1.1.4.1.0 = OID: .1.3.6.1.4.1.18372.3.1.1.1.2.1',
        ]


@pytest.fixture
def snmp_test_params():
    return SNMPTestParams()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import asyncio

from axosyslog_light.common.asynchronous import BackgroundEventLoop


class SingleLineStreamReader(object):
    def __init__(self, stream):
        self._stream = stream

    def wait_for_messages(self, lines, timeout):
        return BackgroundEventLoop().wait_async_result(self._read_until_lines_found(lines, timeout=timeout))

    def wait_for_number_of_messages(self, number_of_lines, timeout):
        return BackgroundEventLoop().wait_async_result(self._read_number_of_lines(number_of_lines, timeout=timeout))

    async def _read_until_lines_found(self, lines, timeout):
        read_lines = []
        lines_to_find = lines.copy()

        while len(lines_to_find) > 0:
            try:
                line = await asyncio.wait_for(self._stream.readline(), timeout=timeout)
            except (asyncio.TimeoutError) as err:
                raise Exception("Could not find all lines. Remaining number of lines to find: {} Number of lines found: {}".format(len(lines_to_find), len(read_lines))) from err

            line = line.decode("utf-8")
            read_lines.append(line)
            _list_remove_partially_matching_element(lines_to_find, line)
        return read_lines

    async def _read_number_of_lines(self, number_of_lines, timeout):
        lines = []
        for i in range(number_of_lines):
            try:
                line = await asyncio.wait_for(self._stream.readline(), timeout=timeout)
            except (asyncio.TimeoutError) as err:
                raise Exception("Could not read {} number of lines. Connection closed after {} lines".format(number_of_lines, len(lines))) from err

            lines.append(line.decode("utf-8"))
        return lines


class DatagramReader(object):
    def __init__(self, datagram_server):
        self._datagram_server = datagram_server

    def wait_for_messages(self, lines, timeout, maxsize=65536):
        return BackgroundEventLoop().wait_async_result(self._read_until_dgrams_found(lines, maxsize, timeout=timeout))

    def wait_for_number_of_messages(self, number_of_msgs, timeout, maxsize=65536):
        return BackgroundEventLoop().wait_async_result(self._read_number_of_dgrams(number_of_msgs, maxsize, timeout=timeout))

    async def _read_until_dgrams_found(self, dgrams, maxsize, timeout):
        read_dgrams = []
        dgrams_to_find = dgrams.copy()

        while len(dgrams_to_find) != 0:
            try:
                dgram = await asyncio.wait_for(self._datagram_server.read_dgram(maxsize), timeout=timeout)
            except (asyncio.TimeoutError) as err:
                raise Exception("Could not find all datagrams. Remaining dgrams to find: {} Lines found: {}".format(len(dgrams_to_find), len(read_dgrams))) from err

            dgram = dgram.decode("utf-8")
            read_dgrams.append(dgram)
            _list_remove_partially_matching_element(dgrams_to_find, dgram)
        return read_dgrams

    async def _read_number_of_dgrams(self, number_of_dgrams, maxsize, timeout):
        msgs = []
        for i in range(number_of_dgrams):
            try:
                msg = await asyncio.wait_for(self._datagram_server.read_dgram(maxsize), timeout=timeout)
            except (asyncio.TimeoutError) as err:
                raise Exception("Could not read {} number of datagrams. Connection closed after {}".format(number_of_dgrams, len(msgs))) from err

            msgs.append(msg.decode("utf-8"))
        return msgs


class FramedStreamReader(object):
    """RFC6587 message reader for the syslog() driver"""
    # TODO msg_length = readuntil(' '); msg = readexactly(msg_length)
    pass


# FileIO and this method too operate on partial string matches
# TODO: LogMessage instances should be used instead, matching the full msg body
def _list_remove_partially_matching_element(list, elem):
    for ll in list:
        if ll in elem:
            list.remove(ll)
            return True
    return False



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from pathlib import Path

from axosyslog_light.common.random_id import get_unique_id


class SyslogNgPaths(object):
    def __init__(self, testcase_parameters):
        self.__testcase_parameters = testcase_parameters
        self.__instance_name = None
        self.__syslog_ng_paths = {}

    def set_syslog_ng_paths(self, instance_name):
        if self.__instance_name is not None:
            raise Exception("Instance already configured")
        self.__instance_name = instance_name
        self.__syslog_ng_paths = {
            "file_paths": {
                "config_path": Path("syslog_ng_{}.conf".format(instance_name)),
                "persist_path": Path("syslog_ng_{}.persist".format(instance_name)),
                "pid_path": Path("syslog_ng_{}.pid".format(instance_name)),
                "control_socket_path": Path("syslog_ng_{}.ctl".format(instance_name)),
                "stderr": Path("syslog_ng_{}_stderr".format(instance_name)),
                "stdout": Path("syslog_ng_{}_stdout".format(instance_name)),
                "syntax_only_stderr": Path("syslog_ng_{}_syntax_only_stderr".format(instance_name)),
                "syntax_only_stdout": Path("syslog_ng_{}_syntax_only_stdout".format(instance_name)),
            },
        }

        try:
            install_dir = self.__testcase_parameters.get_install_dir()
            self.__syslog_ng_paths.update(
                {
                    "dirs": {"install_dir": install_dir},
                    "binary_file_paths": {
                        "slogkey": Path(install_dir, "bin", "slogkey"),
                        "slogverify": Path(install_dir, "bin", "slogverify"),
                        "syslog_ng_binary": Path(install_dir, "sbin", "syslog-ng"),
                        "syslog_ng_ctl": Path(install_dir, "sbin", "syslog-ng-ctl"),
                        "loggen": Path(install_dir, "bin", "loggen"),
                    },
                },
            )
        except KeyError:
            pass

        return self

    def get_instance_name(self):
        return self.__instance_name

    def get_install_dir(self):
        return self.__syslog_ng_paths["dirs"]["install_dir"]

    def get_config_path(self):
        return self.__syslog_ng_paths["file_paths"]["config_path"]

    def get_persist_path(self):
        return self.__syslog_ng_paths["file_paths"]["persist_path"]

    def get_pid_path(self):
        return self.__syslog_ng_paths["file_paths"]["pid_path"]

    def get_control_socket_path(self):
        return self.__syslog_ng_paths["file_paths"]["control_socket_path"]

    def get_stderr_path(self):
        return self.__syslog_ng_paths["file_paths"]["stderr"]

    def get_stderr_path_with_postfix(self, postfix):
        return Path("{}_{}".format(self.__syslog_ng_paths["file_paths"]["stderr"], postfix))

    def get_stdout_path(self):
        return self.__syslog_ng_paths["file_paths"]["stdout"]

    def get_stdout_path_with_postfix(self, postfix):
        return Path("{}_{}".format(self.__syslog_ng_paths["file_paths"]["stdout"], postfix))

    def get_syntax_only_stderr_path(self):
        return self.__syslog_ng_paths["file_paths"]["syntax_only_stderr"]

    def get_syntax_only_stdout_path(self):
        return self.__syslog_ng_paths["file_paths"]["syntax_only_stdout"]

    def get_syslog_ng_bin(self):
        return self.__syslog_ng_paths["binary_file_paths"]["syslog_ng_binary"]

    def get_slogkey_bin(self):
        return self.__syslog_ng_paths["binary_file_paths"]["slogkey"]

    def get_slogverify_bin(self):
        return self.__syslog_ng_paths["binary_file_paths"]["slogverify"]

    def get_syslog_ng_ctl_bin(self):
        return self.__syslog_ng_paths["binary_file_paths"]["syslog_ng_ctl"]

    def get_loggen_bin(self):
        return self.__syslog_ng_paths["binary_file_paths"]["loggen"]

    def register_external_tool_output_path(self, external_tool):
        self.__syslog_ng_paths['file_paths'].update(
            {
                external_tool: Path("{}_{}.log".format(external_tool, get_unique_id())),
            },
        )

    def get_external_tool_output_path(self, external_tool):
        return self.__syslog_ng_paths['file_paths'][external_tool]



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
# flake8: noqa: F401, F811
from pathlib import PosixPath

import pytest
from axosyslog_light.self_test_fixtures import fake_testcase_parameters
from axosyslog_light.syslog_ng.syslog_ng_paths import SyslogNgPaths


def test_syslog_ng_paths(fake_testcase_parameters):
    syslog_ng_paths = SyslogNgPaths(fake_testcase_parameters)
    syslog_ng_paths.set_syslog_ng_paths(instance_name="server")
    assert set(list(syslog_ng_paths._SyslogNgPaths__syslog_ng_paths)) == {"dirs", "file_paths", "binary_file_paths"}
    assert set(list(syslog_ng_paths._SyslogNgPaths__syslog_ng_paths["dirs"])) == {"install_dir"}
    assert set(list(syslog_ng_paths._SyslogNgPaths__syslog_ng_paths["file_paths"])) == {
        "config_path",
        "persist_path",
        "pid_path",
        "control_socket_path",
        "stderr",
        "stdout",
        "syntax_only_stderr",
        "syntax_only_stdout",
    }
    assert set(list(syslog_ng_paths._SyslogNgPaths__syslog_ng_paths["binary_file_paths"])) == {
        "syslog_ng_binary",
        "syslog_ng_ctl",
        "loggen",
        "slogkey",
        "slogverify",
    }


def test_syslog_ng_paths_parent_class_of_paths(fake_testcase_parameters):
    syslog_ng_paths = SyslogNgPaths(fake_testcase_parameters)
    syslog_ng_paths.set_syslog_ng_paths(instance_name="server")
    for __key, value in syslog_ng_paths._SyslogNgPaths__syslog_ng_paths["file_paths"].items():
        assert isinstance(value, PosixPath) is True

    for __key, value in syslog_ng_paths._SyslogNgPaths__syslog_ng_paths["dirs"].items():
        assert isinstance(value, PosixPath) is True

    for __key, value in syslog_ng_paths._SyslogNgPaths__syslog_ng_paths["binary_file_paths"].items():
        assert isinstance(value, PosixPath) is True


def test_syslog_ng_paths_client_relay_server(fake_testcase_parameters):
    syslog_ng_paths_server = SyslogNgPaths(
        fake_testcase_parameters,
    ).set_syslog_ng_paths(instance_name="server")
    syslog_ng_paths_relay = SyslogNgPaths(
        fake_testcase_parameters,
    ).set_syslog_ng_paths(instance_name="relay")
    syslog_ng_paths_client = SyslogNgPaths(
        fake_testcase_parameters,
    ).set_syslog_ng_paths(instance_name="client")

    assert syslog_ng_paths_client.get_instance_name() == "client"
    assert syslog_ng_paths_relay.get_instance_name() == "relay"
    assert syslog_ng_paths_server.get_instance_name() == "server"


def test_instance_already_configured(fake_testcase_parameters):
    syslog_ng_paths_server = SyslogNgPaths(
        fake_testcase_parameters,
    ).set_syslog_ng_paths(instance_name="server")
    with pytest.raises(Exception):
        syslog_ng_paths_server.set_syslog_ng_paths(instance_name="client")



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import typing
from dataclasses import dataclass
from dataclasses import field

from grpc import insecure_channel
from opentelemetry.proto.collector.logs.v1.logs_service_pb2 import ExportLogsServiceRequest
from opentelemetry.proto.collector.logs.v1.logs_service_pb2_grpc import LogsServiceStub
from opentelemetry.proto.common.v1.common_pb2 import AnyValue
from opentelemetry.proto.common.v1.common_pb2 import ArrayValue
from opentelemetry.proto.common.v1.common_pb2 import InstrumentationScope
from opentelemetry.proto.common.v1.common_pb2 import KeyValue
from opentelemetry.proto.common.v1.common_pb2 import KeyValueList
from opentelemetry.proto.logs.v1.logs_pb2 import LogRecord
from opentelemetry.proto.logs.v1.logs_pb2 import ResourceLogs
from opentelemetry.proto.logs.v1.logs_pb2 import ScopeLogs
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_DEBUG
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_DEBUG2
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_DEBUG3
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_DEBUG4
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_ERROR
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_ERROR2
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_ERROR3
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_ERROR4
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_FATAL
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_FATAL2
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_FATAL3
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_FATAL4
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_INFO
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_INFO2
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_INFO3
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_INFO4
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_TRACE
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_TRACE2
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_TRACE3
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_TRACE4
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_UNSPECIFIED
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_WARN
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_WARN2
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_WARN3
from opentelemetry.proto.logs.v1.logs_pb2 import SEVERITY_NUMBER_WARN4
from opentelemetry.proto.resource.v1.resource_pb2 import Resource


class PyToOTelConverter:
    @staticmethod
    def convert_value_to_any_value(value: typing.Any) -> AnyValue:
        if isinstance(value, str):
            return AnyValue(string_value=value)
        if isinstance(value, bool):
            return AnyValue(bool_value=value)
        if isinstance(value, int):
            return AnyValue(int_value=value)
        if isinstance(value, float):
            return AnyValue(double_value=value)
        if isinstance(value, bytes):
            return AnyValue(bytes_value=value)
        if isinstance(value, dict):
            return AnyValue(kvlist_value=PyToOTelConverter.convert_dict_to_key_value_list(value))
        if isinstance(value, list):
            return AnyValue(array_value=PyToOTelConverter.convert_list_to_values(value))
        if isinstance(value, type(None)):
            return AnyValue()
        raise ValueError("Unsupported value type: {}".format(type(value)))

    @staticmethod
    def convert_dict_to_key_value_list(data: typing.Dict[str, typing.Any]) -> KeyValueList:
        key_value_list = KeyValueList()

        for key, value in data.items():
            if not isinstance(key, str):
                raise ValueError("Key must be a string: {}".format(key))

            key_value = KeyValue()
            key_value.key = key
            key_value.value.CopyFrom(PyToOTelConverter.convert_value_to_any_value(value))
            key_value_list.values.append(key_value)

        return key_value_list

    @staticmethod
    def convert_list_to_values(data: typing.List[typing.Any]) -> ArrayValue:
        array_value = ArrayValue()

        for value in data:
            array_value.values.append(PyToOTelConverter.convert_value_to_any_value(value))

        return array_value


@dataclass
class OTelResource:
    attributes: typing.Dict[str, typing.Any] = field(default_factory=dict)

    def to_otel(self) -> Resource:
        resource = Resource()
        resource.attributes.extend(PyToOTelConverter.convert_dict_to_key_value_list(self.attributes).values)
        return resource


@dataclass
class OTelScope:
    name: str = ""
    version: str = ""
    attributes: typing.Dict[str, typing.Any] = field(default_factory=dict)

    def to_otel(self) -> InstrumentationScope:
        scope = InstrumentationScope()
        scope.name = self.name
        scope.version = self.version
        scope.attributes.extend(PyToOTelConverter.convert_dict_to_key_value_list(self.attributes).values)
        return scope


@dataclass
class OTelLog:
    time_unix_nano: int = 0
    observed_time_unix_nano: int = 0
    severity_number: int = 0
    severity_text: str = ""
    body: typing.Any = None
    attributes: typing.Dict[str, typing.Any] = field(default_factory=dict)
    flags: int = 0
    trace_id: bytes = b""
    span_id: bytes = b""

    def to_otel(self) -> LogRecord:
        SEVERITY_NUMBERS = [
            SEVERITY_NUMBER_UNSPECIFIED,
            SEVERITY_NUMBER_TRACE,
            SEVERITY_NUMBER_TRACE2,
            SEVERITY_NUMBER_TRACE3,
            SEVERITY_NUMBER_TRACE4,
            SEVERITY_NUMBER_DEBUG,
            SEVERITY_NUMBER_DEBUG2,
            SEVERITY_NUMBER_DEBUG3,
            SEVERITY_NUMBER_DEBUG4,
            SEVERITY_NUMBER_INFO,
            SEVERITY_NUMBER_INFO2,
            SEVERITY_NUMBER_INFO3,
            SEVERITY_NUMBER_INFO4,
            SEVERITY_NUMBER_WARN,
            SEVERITY_NUMBER_WARN2,
            SEVERITY_NUMBER_WARN3,
            SEVERITY_NUMBER_WARN4,
            SEVERITY_NUMBER_ERROR,
            SEVERITY_NUMBER_ERROR2,
            SEVERITY_NUMBER_ERROR3,
            SEVERITY_NUMBER_ERROR4,
            SEVERITY_NUMBER_FATAL,
            SEVERITY_NUMBER_FATAL2,
            SEVERITY_NUMBER_FATAL3,
            SEVERITY_NUMBER_FATAL4,
        ]

        log = LogRecord()
        log.time_unix_nano = self.time_unix_nano
        log.observed_time_unix_nano = self.observed_time_unix_nano
        log.severity_number = SEVERITY_NUMBERS[self.severity_number]
        log.severity_text = self.severity_text
        log.body.CopyFrom(PyToOTelConverter.convert_value_to_any_value(self.body))
        log.attributes.extend(PyToOTelConverter.convert_dict_to_key_value_list(self.attributes).values)
        log.flags = self.flags
        log.trace_id = self.trace_id
        log.span_id = self.span_id
        return log


class OTelResourceScopeLog:
    def __init__(
        self,
        resource: typing.Optional[OTelResource] = None,
        scope: typing.Optional[OTelScope] = None,
        log: typing.Optional[OTelLog] = None,
    ) -> None:
        self.resource = resource if resource is not None else OTelResource()
        self.scope = scope if scope is not None else OTelScope()
        self.log = log if log is not None else OTelLog()


class OpenTelemetryIO():
    def __init__(self, port: int) -> None:
        self.__port = port

    def __create_request(self, resource_scope_logs: typing.List[OTelResourceScopeLog]) -> ExportLogsServiceRequest:
        request = ExportLogsServiceRequest()
        for resource_scope_log in resource_scope_logs:
            resource_logs = ResourceLogs()
            resource_logs.resource.CopyFrom(resource_scope_log.resource.to_otel())

            new_resource_logs = True
            for stored_resource_logs in request.resource_logs:
                if stored_resource_logs.resource == resource_logs.resource:
                    resource_logs = stored_resource_logs
                    new_resource_logs = False
                    break
            if new_resource_logs:
                request.resource_logs.append(resource_logs)
                resource_logs = request.resource_logs[-1]

            scope_logs = ScopeLogs()
            scope_logs.scope.CopyFrom(resource_scope_log.scope.to_otel())

            new_scope_logs = True
            for stored_scope_logs in resource_logs.scope_logs:
                if stored_scope_logs.scope == scope_logs.scope:
                    scope_logs = stored_scope_logs
                    new_scope_logs = False
                    break
            if new_scope_logs:
                resource_logs.scope_logs.append(scope_logs)
                scope_logs = resource_logs.scope_logs[-1]

            log_record = resource_scope_log.log.to_otel()
            scope_logs.log_records.append(log_record)

        return request

    def send_logs(self, resource_scope_logs: typing.List[OTelResourceScopeLog]) -> None:
        request = self.__create_request(resource_scope_logs)
        with insecure_channel(f"localhost:{self.__port}") as channel:
            stub = LogsServiceStub(channel)
            stub.Export(request)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import typing


class LogContextSclBlock:
    group_type = "log_context_scl_block"

    def __init__(self, name: str, **options: typing.Dict[str, typing.Any]) -> None:
        self.__name = name
        self.__options = options

    @property
    def name(self):
        return self.__name

    @property
    def options(self):
        return self.__options



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
# Copyright (c) 2015-2018 Balabit
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import shlex
import typing
from pathlib import Path
from subprocess import Popen

from axosyslog_light.common.file import copy_shared_file
from axosyslog_light.executors.command_executor import CommandExecutor
from axosyslog_light.executors.process_executor import ProcessExecutor
from axosyslog_light.syslog_ng.syslog_ng_executor import SyslogNgExecutor
from axosyslog_light.syslog_ng.syslog_ng_executor import SyslogNgStartParams
from axosyslog_light.testcase_parameters.testcase_parameters import TestcaseParameters


class SyslogNgLocalExecutor(SyslogNgExecutor):
    def __init__(
        self,
        syslog_ng_binary_path: Path,
        tc_parameters: TestcaseParameters,
    ) -> None:
        self.__process_executor = ProcessExecutor()
        self.__command_executor = CommandExecutor()
        self.__syslog_ng_binary_path = syslog_ng_binary_path
        self.__tc_parameters = tc_parameters

    def run_process(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
    ) -> Popen:
        return self.__process_executor.start(
            command=self.__construct_syslog_ng_command(start_params),
            stdout_path=stdout_path,
            stderr_path=stderr_path,
        )

    def run_process_with_valgrind(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
        valgrind_output_path: Path,
    ) -> Popen:
        valgrind_command_args = [
            "valgrind",
            "--show-leak-kinds=all",
            "--track-origins=yes",
            "--tool=memcheck",
            "--leak-check=full",
            "--keep-stacktraces=alloc-and-free",
            "--read-var-info=yes",
            "--error-limit=no",
            "--num-callers=40",
            "--verbose",
            f"--log-file={str(valgrind_output_path)}",
        ]
        full_command_args = valgrind_command_args + self.__construct_syslog_ng_command(start_params)
        return self.__process_executor.start(
            command=full_command_args,
            stdout_path=stdout_path,
            stderr_path=stderr_path,
        )

    def run_process_with_gdb(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
    ) -> Popen:
        gdb_command_args = [
            "gdb",
            "-ex",
            f"r {shlex.join(start_params.format())} > {str(stdout_path)} 2> {str(stderr_path)}",
            str(self.__syslog_ng_binary_path),
        ]
        return self.__process_executor.start(
            command=["xterm", "-fa", "Monospace", "-fs", "18", "-e", shlex.join(gdb_command_args)],
            stdout_path="/dev/null",
            stderr_path="/dev/null",
        )

    def run_process_with_gdb_for_bt(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
    ) -> Popen:
        gdb_exec_file_name = "gdb_exec.sh"
        copy_shared_file(self.__tc_parameters, gdb_exec_file_name)
        gdb_exec_cmd = [
            f'./{gdb_exec_file_name}',
            f'"{self.__syslog_ng_binary_path}"',
            f'{shlex.join(start_params.format())}',
        ]
        return self.__command_executor.run(
            command=gdb_exec_cmd,
            stdout_path="gdb_stdout.log",
            stderr_path="gdb_stderr.log",
        )

    def run_process_with_strace(
        self,
        start_params: SyslogNgStartParams,
        stderr_path: Path,
        stdout_path: Path,
        strace_output_path: Path,
    ) -> Popen:
        strace_command_args = [
            "strace",
            "-s",
            "4096",
            "-tt",
            "-T",
            "-ff",
            "-o",
            str(strace_output_path),
        ]
        full_command_args = strace_command_args + self.__construct_syslog_ng_command(start_params)
        return self.__process_executor.start(
            command=full_command_args,
            stdout_path=stdout_path,
            stderr_path=stderr_path,
        )

    def get_backtrace_from_core(
        self,
        core_file_path: Path,
        stderr_path: Path,
        stdout_path: Path,
    ) -> typing.Dict[str, typing.Any]:
        gdb_command_args = [
            "gdb",
            "-ex",
            "bt full",
            "--batch",
            str(self.__syslog_ng_binary_path),
            "--core",
            str(core_file_path),
        ]
        return self.__command_executor.run(
            command=gdb_command_args,
            stdout_path=stdout_path,
            stderr_path=stderr_path,
        )

    def __construct_syslog_ng_command(
        self,
        start_params: typing.Optional[SyslogNgStartParams] = None,
    ) -> typing.List[str]:
        return [str(self.__syslog_ng_binary_path)] + start_params.format()
